@@ -26,6 +26,7 @@
 #include <glib.h>
 #include <glib-object.h>
 #include <pango/pango.h>
+#include <hb-ot.h>
 
 G_BEGIN_DECLS
 
@@ -128,8 +129,46 @@ pango_glyph_string_reverse_range (PangoGlyphString *glyphs,
     }
 }
 
-/* The cairo hexbox drawing code assumes
- * that these nicks are 1-6 ASCII chars
+static inline gboolean
+pango_is_default_ignorable (gunichar ch)
+{
+  int plane = ch >> 16;
+
+  if (G_LIKELY (plane == 0))
+    {
+      int page = ch >> 8;
+      switch (page)
+        {
+        case 0x00: return ch == 0x00ad;
+        case 0x03: return ch == 0x034f;
+        case 0x06: return ch == 0x061c;
+        case 0x17: return (0x17b4 <= ch && ch <= 0x17b5);
+        case 0x18: return (0x180b <= ch && ch <= 0x180e);
+        case 0x20: return (0x200b <= ch && ch <= 0x200f) ||
+                          (0x202a <= ch && ch <= 0x202e) ||
+                          (0x2060 <= ch && ch <= 0x206f);
+        case 0xfe: return (0xfe00 <= ch && ch <= 0xfe0f) || ch == 0xfeff;
+        case 0xff: return (0xfff0 <= ch && ch <= 0xfff8);
+        default: return FALSE;
+      }
+    }
+  else
+    {
+      /* Other planes */
+      switch (plane)
+        {
+        case 0x01: return (0x1d173 <= ch && ch <= 0x1d17a);
+        case 0x0e: return (0xe0000 <= ch && ch <= 0xe0fff);
+        default: return FALSE;
+        }
+    }
+}
+
+/* These are the default ignorables that we render as hexboxes
+ * with nicks if PANGO_SHOW_IGNORABLES is used.
+ *
+ * The cairo hexbox drawing code assumes that these nicks are
+ * 1-6 ASCII chars
  */
 static struct {
   gunichar ch;
@@ -217,6 +256,37 @@ pango_get_ignorable_size (gunichar  ch,
 # define g_memdup2(mem,size)    g_memdup((mem),(size))
 #endif
 
+static inline void
+pango_parse_variations (const char            *variations,
+                        hb_ot_var_axis_info_t *axes,
+                        int                    n_axes,
+                        float                 *coords)
+{
+  const char *p;
+  const char *end;
+  hb_variation_t var;
+  int i;
+
+  p = variations;
+  while (p && *p)
+    {
+      end = strchr (p, ',');
+      if (hb_variation_from_string (p, end ? end - p: -1, &var))
+        {
+          for (i = 0; i < n_axes; i++)
+            {
+              if (axes[i].tag == var.tag)
+                {
+                  coords[axes[i].axis_index] = var.value;
+                  break;
+                }
+            }
+        }
+
+      p = end ? end + 1 : NULL;
+    }
+}
+
 G_END_DECLS
 
 #endif /* __PANGO_IMPL_UTILS_H__ */
