@@ -653,7 +653,8 @@ add_run (GtkJsonPrinter *printer,
   gtk_json_printer_add_string (printer, "language", pango_language_to_string (run->item->analysis.language));
   gtk_json_printer_add_string (printer, "script", get_script_name (run->item->analysis.script));
 
-  add_font (printer, "font", run->item->analysis.font);
+  if (run->item->analysis.font)
+    add_font (printer, "font", run->item->analysis.font);
 
   gtk_json_printer_add_integer (printer, "flags", run->item->analysis.flags & ANALYSIS_FLAGS);
 
@@ -742,7 +743,7 @@ add_output (GtkJsonPrinter *printer,
 
   pango_layout_get_size (layout, &width, &height);
   gtk_json_printer_add_integer (printer, "width", width);
-  gtk_json_printer_add_integer (printer, "height", width);
+  gtk_json_printer_add_integer (printer, "height", height);
 
   add_log_attrs (printer, layout);
   gtk_json_printer_start_array (printer, "lines");
@@ -1712,7 +1713,7 @@ pango_layout_deserialize (PangoContext                 *context,
         code = PANGO_LAYOUT_DESERIALIZE_INVALID;
 
       g_set_error (error, PANGO_LAYOUT_DESERIALIZE_ERROR, code,
-                   "%ld:%ld: %s", start, end, parser_error->message);
+                   "%" G_GSIZE_FORMAT ":%" G_GSIZE_FORMAT ": %s", start, end, parser_error->message);
 
       g_clear_object (&layout);
     }
