@@ -19,6 +19,8 @@
  * Boston, MA 02111-1307, USA.
  */
 
+#include "config.h"
+
 #include <glib.h>
 #include <string.h>
 #include <locale.h>
@@ -26,6 +28,13 @@
 #include <gio/gio.h>
 #include <pango/pangocairo.h>
 
+#include "test-common.h"
+
+#ifdef HAVE_FREETYPE
+#include <pango/pangoft2.h>
+#include <fontconfig/fontconfig.h>
+#endif
+
 static void
 test_parse (void)
 {
@@ -238,7 +247,7 @@ test_enumerate (void)
   PangoFont *font;
   gboolean found_face;
 
-  fontmap = pango_cairo_font_map_new ();
+  fontmap = get_font_map_with_cantarell ();
   context = pango_font_map_create_context (fontmap);
 
   pango_font_map_list_families (fontmap, &families, &n_families);
@@ -297,9 +306,68 @@ test_roundtrip_plain (void)
   desc = pango_font_description_from_string ("Cantarell 11");
 #endif
 
-  fontmap = pango_cairo_font_map_new ();
+  fontmap = get_font_map_with_cantarell ();
   context = pango_font_map_create_context (fontmap);
 
+  font = pango_context_load_font (context, desc);
+  desc2 = pango_font_describe (font);
+
+  g_assert_true (pango_font_description_equal (desc2, desc));
+
+  pango_font_description_free (desc2);
+  g_object_unref (font);
+  pango_font_description_free (desc);
+  g_object_unref (context);
+  g_object_unref (fontmap);
+}
+
+static void
+test_roundtrip_absolute (void)
+{
+  PangoFontMap *fontmap;
+  PangoContext *context;
+  PangoFontDescription *desc, *desc2;
+  PangoFont *font;
+
+#ifdef HAVE_CARBON
+  desc = pango_font_description_from_string ("Helvetica 11");
+#else
+  desc = pango_font_description_from_string ("Cantarell 11");
+#endif
+
+  pango_font_description_set_absolute_size (desc, 11 * 1024.0);
+
+  fontmap = get_font_map_with_cantarell ();
+  context = pango_font_map_create_context (fontmap);
+
+  font = pango_context_load_font (context, desc);
+  desc2 = pango_font_describe_with_absolute_size (font);
+
+  g_assert_true (pango_font_description_equal (desc2, desc));
+
+  pango_font_description_free (desc2);
+  g_object_unref (font);
+  pango_font_description_free (desc);
+  g_object_unref (context);
+  g_object_unref (fontmap);
+}
+
+static void
+test_roundtrip_variations (void)
+{
+  PangoFontMap *fontmap;
+  PangoContext *context;
+  PangoFontDescription *desc, *desc2;
+  PangoFont *font;
+
+#ifdef HAVE_CARBON
+  desc = pango_font_description_from_string ("Helvetica 11 @wght=444");
+#else
+  desc = pango_font_description_from_string ("Cantarell 11 @wght=444");
+#endif
+
+  fontmap = get_font_map_with_cantarell ();
+  context = pango_font_map_create_context (fontmap);
 
   font = pango_context_load_font (context, desc);
   desc2 = pango_font_describe (font);
@@ -324,9 +392,11 @@ test_roundtrip_small_caps (void)
   guint num = 0;
 
   fontmap = pango_cairo_font_map_new ();
-  if (strcmp (G_OBJECT_TYPE_NAME (fontmap), "PangoCairoCoreTextFontMap") == 0)
+  if (strcmp (G_OBJECT_TYPE_NAME (fontmap), "PangoCairoCoreTextFontMap") == 0 ||
+      strcmp (G_OBJECT_TYPE_NAME (fontmap), "PangoCairoWin32FontMap") == 0)
     {
-      g_test_skip ("Small Caps support needs to be added to PangoCoreTextFontMap");
+      g_test_incomplete_printf ("Small Caps support needs to be added to %s",
+                                G_OBJECT_TYPE_NAME (fontmap));
       g_object_unref (fontmap);
       return;
     }
@@ -487,8 +557,20 @@ test_font_models (void)
 
   g_assert_true (monospace_found);
 
-  g_print ("# %d font families, %d monospace, %d variable\n",
-           n_families, n_monospace_families, n_variable_families);
+  if (g_test_verbose ())
+    {
+      g_test_message ("# %d font families, %d monospace, %d variable",
+                      n_families, n_monospace_families, n_variable_families);
+
+      for (guint i = 0; i < g_list_model_get_n_items (G_LIST_MODEL (map)); i++)
+        {
+          GObject *obj = g_list_model_get_item (G_LIST_MODEL (map), i);
+
+          g_test_message ("%s", pango_font_family_get_name (PANGO_FONT_FAMILY (obj)));
+
+          g_object_unref (obj);
+        }
+    }
 
   g_object_unref (map);
 }
@@ -585,7 +667,8 @@ test_font_scale (void)
   cairo_scaled_font_t *sf;
   cairo_font_options_t *options2;
 
-  fontmap = pango_cairo_font_map_new ();
+  fontmap = get_font_map_with_cantarell ();
+
   context = pango_font_map_create_context (fontmap);
 
   options = cairo_font_options_create ();
@@ -595,15 +678,38 @@ test_font_scale (void)
 
   pango_cairo_context_set_font_options (context, options);
 
-  desc = pango_font_description_from_string ("Cantarell Small-Caps 11 @wght=444");
+  desc = pango_font_description_from_string ("Cantarell 11");
 
   font = pango_font_map_load_font (fontmap, context, desc);
 
+  if (strcmp (pango_font_family_get_name (pango_font_face_get_family (pango_font_get_face (font))), "Cantarell") != 0)
+    {
+      g_test_skip ("Fontmap has no Cantarell");
+
+      pango_font_description_free (desc);
+      cairo_font_options_destroy (options);
+      g_object_unref (context);
+      g_object_unref (fontmap);
+
+      return;
+    }
+
   scaled_font = pango_font_map_reload_font (fontmap, font, 1.5, NULL, NULL);
 
+  g_assert_true (scaled_font != font);
+
   scaled_desc = pango_font_describe (scaled_font);
+
+  /* FIXME there are rounding errors in the win32 font map code, so we have
+   * to compare the sizes with some slop
+   */
+  g_assert_cmpfloat_with_epsilon (16.5, pango_font_description_get_size (scaled_desc) / (double) PANGO_SCALE, 0.005);
+
+  pango_font_description_set_size (scaled_desc, 16.5 * 1024);
+
   str = pango_font_description_to_string (scaled_desc);
-  g_assert_cmpstr (str, ==, "Cantarell Small-Caps 16.5 @wght=444");
+
+  g_assert_cmpstr (str, ==, "Cantarell 16.5");
 
   /* check that we also preserve font options */
   options2 = cairo_font_options_create ();
@@ -624,9 +730,11 @@ test_font_scale (void)
 
   scaled_font = pango_font_map_reload_font (fontmap, font, 1, context, NULL);
 
+  g_assert_true (scaled_font != font);
+
   scaled_desc = pango_font_describe (scaled_font);
   str = pango_font_description_to_string (scaled_desc);
-  g_assert_cmpstr (str, ==, "Cantarell Small-Caps 11 @wght=444");
+  g_assert_cmpstr (str, ==, "Cantarell 11");
 
   sf = pango_cairo_font_get_scaled_font (PANGO_CAIRO_FONT (scaled_font));
   cairo_scaled_font_get_font_options (sf, options2);
@@ -639,13 +747,56 @@ test_font_scale (void)
   pango_font_description_free (scaled_desc);
   g_free (str);
 
-  /* Try again, this time with different variations */
+  g_object_unref (font);
+  pango_font_description_free (desc);
+  g_object_unref (context);
+  g_object_unref (fontmap);
+}
+
+static void
+test_font_scale_variations (void)
+{
+  PangoFontMap *fontmap;
+  PangoContext *context;
+  PangoFontDescription *desc;
+  PangoFont *font;
+  PangoFont *scaled_font;
+  PangoFontDescription *scaled_desc;
+  char *str;
+  cairo_font_options_t *options;
+
+  fontmap = get_font_map_with_cantarell ();
+
+  context = pango_font_map_create_context (fontmap);
+
+  options = cairo_font_options_create ();
+  cairo_font_options_set_antialias (options, CAIRO_ANTIALIAS_NONE);
+  cairo_font_options_set_hint_style (options, CAIRO_HINT_STYLE_FULL);
+  cairo_font_options_set_hint_metrics (options, CAIRO_HINT_METRICS_ON);
+
+  pango_cairo_context_set_font_options (context, options);
+
+  desc = pango_font_description_from_string ("Cantarell 11");
+
+  font = pango_font_map_load_font (fontmap, context, desc);
+
+  if (strcmp (pango_font_family_get_name (pango_font_face_get_family (pango_font_get_face (font))), "Cantarell") != 0)
+    {
+      g_test_skip ("Fontmap has no Cantarell");
+
+      pango_font_description_free (desc);
+      cairo_font_options_destroy (options);
+      g_object_unref (context);
+      g_object_unref (fontmap);
+
+      return;
+    }
 
   scaled_font = pango_font_map_reload_font (fontmap, font, 1, NULL, "wght=666");
 
   scaled_desc = pango_font_describe (scaled_font);
   str = pango_font_description_to_string (scaled_desc);
-  g_assert_cmpstr (str, ==, "Cantarell Small-Caps 11 @wght=666");
+  g_assert_cmpstr (str, ==, "Cantarell 11 @wght=666");
 
   g_object_unref (scaled_font);
   pango_font_description_free (scaled_desc);
@@ -657,12 +808,66 @@ test_font_scale (void)
   g_object_unref (fontmap);
 }
 
+static void
+test_font_custom (void)
+{
+  PangoFontMap *fontmap;
+  char *path;
+  GError *error = NULL;
+
+  fontmap = pango_cairo_font_map_new ();
+
+  if (strcmp (G_OBJECT_TYPE_NAME (fontmap), "PangoCairoCoreTextFontMap") == 0)
+    {
+      g_test_incomplete_printf ("%s does not support adding custom fonts",
+                                G_OBJECT_TYPE_NAME (fontmap));
+      return;
+    }
+
+  path = g_test_build_filename (G_TEST_DIST, "fonts", "Cantarell-VF.otf", NULL);
+  if (g_test_verbose ())
+    g_test_message ("adding %s to font map", path);
+  pango_font_map_add_font_file (fontmap, path, &error);
+  g_assert_no_error (error);
+
+  /* Check that we got our Cantarell, not the system one.
+   * FIXME: do a similar check for dwrite
+   */
+#ifdef HAVE_FREETYPE
+  if (strcmp (G_OBJECT_TYPE_NAME (fontmap), "PangoCairoFcFontMap") == 0)
+    {
+      PangoContext *context;
+      PangoFontDescription *desc;
+      PangoFont *font;
+      FcPattern *pattern;
+      const char *filename;
+
+      context = pango_font_map_create_context (fontmap);
+      desc = pango_font_description_from_string ("Cantarell 11");
+      font = pango_font_map_load_font (fontmap, context, desc);
+
+      pattern = pango_fc_font_get_pattern (PANGO_FC_FONT (font));
+      if (FcPatternGetString (pattern, FC_FILE, 0, (FcChar8 **)&filename) != FcResultMatch)
+        filename = "tough luck";
+
+      g_assert_cmpstr (filename, ==, path);
+
+      g_object_unref (font);
+      pango_font_description_free (desc);
+      g_object_unref (context);
+    }
+#endif
+
+  g_free (path);
+}
+
 int
 main (int argc, char *argv[])
 {
   setlocale (LC_ALL, "");
 
   g_test_init (&argc, &argv, NULL);
+  g_test_set_nonfatal_assertions ();
 
   g_test_add_func ("/pango/font/metrics", test_metrics);
   g_test_add_func ("/pango/fontdescription/parse", test_parse);
@@ -675,12 +880,16 @@ main (int argc, char *argv[])
   g_test_add_func ("/pango/font/extents", test_extents);
   g_test_add_func ("/pango/font/enumerate", test_enumerate);
   g_test_add_func ("/pango/font/roundtrip/plain", test_roundtrip_plain);
+  g_test_add_func ("/pango/font/roundtrip/absolute", test_roundtrip_absolute);
+  g_test_add_func ("/pango/font/roundtrip/variations", test_roundtrip_variations);
   g_test_add_func ("/pango/font/roundtrip/small-caps", test_roundtrip_small_caps);
   g_test_add_func ("/pango/font/roundtrip/emoji", test_roundtrip_emoji);
   g_test_add_func ("/pango/font/models", test_font_models);
   g_test_add_func ("/pango/font/glyph-extents", test_glyph_extents);
   g_test_add_func ("/pango/font/font-metrics", test_font_metrics);
-  g_test_add_func ("/pango/font/scale-font", test_font_scale);
+  g_test_add_func ("/pango/font/scale-font/plain", test_font_scale);
+  g_test_add_func ("/pango/font/scale-font/variations", test_font_scale_variations);
+  g_test_add_func ("/pango/font/custom", test_font_custom);
 
   return g_test_run ();
 }
